=== modified file 'NEWS'
--- NEWS	2009-07-17 11:20:32 +0000
+++ NEWS	2009-07-24 18:07:59 +0000
@@ -1,5 +1,16 @@
 bzr-svn 0.6.4					 UNRELEASED
 
+  BUG FIXES
+
+   * Cope with http exceptions in the bzr-svn probe code a bit more 
+     gracefully.
+
+ FEATURES
+
+   * Implement custom "bzr send" format that matches the format used by "svn diff". 
+     This is used by default when submitting diffs against Subversion branches.
+	 (Lukas Lalinský, Jelmer Vernooij)
+
 bzr-svn 0.6.3                    2009-07-17
 
  FEATURES

=== modified file 'branch.py'
--- branch.py	2009-07-22 13:14:10 +0000
+++ branch.py	2009-07-24 18:07:59 +0000
@@ -237,7 +237,7 @@
         """
         return self.last_revmeta()[0].revnum
 
-   def get_child_submit_format(self):
+    def get_child_submit_format(self):
         """Return the preferred format of submissions to this branch."""
         ret = self.get_config().get_user_option("child_submit_format")
         if ret is not None:

=== modified file 'send.py'
--- send.py	2009-07-22 11:11:01 +0000
+++ send.py	2009-07-24 18:07:59 +0000
@@ -17,55 +17,128 @@
 import time
 from bzrlib import (
     branch as _mod_branch,
+    diff as _mod_diff,
     errors,
     merge_directive,
     osutils,
-    revision as _mod_revision
-    )
-
-
-class SvnDiffWriter(object):
-
-    def __init__(self, repository, base_revision_id):
-        self.repository = repository
-        self.base_revision_id = base_revision_id
-        self.tree_rev_info = {}
-
-    def get_svn_rev_info(self, tree):
-        if tree in self.tree_rev_info:
-            return self.tree_rev_info[tree]
-        revision_id = tree.get_revision_id()
-        if revision_id == self.base_revision_id:
-            rev_info = '(working copy)'
-        elif _mod_revision.is_null(revision_id):
-            rev_info = '(revision 0)'
-        else:
-            info = self.repository.lookup_revision_id(revision_id)
-            rev_info = '(revision %d)' % info[0][2]
-        self.tree_rev_info[tree] = rev_info
-        return rev_info
-
-    def diff_text(self, difftext, file_id, old_path, new_path, old_kind, new_kind):
-        if 'file' not in (old_kind, new_kind):
-            return difftext.CANNOT_DIFF
-        from_file_id = to_file_id = file_id
-        if old_kind == 'file':
-            old_date = self.get_svn_rev_info(difftext.old_tree)
-        elif old_kind is None:
-            old_date = None
-            from_file_id = None
-        else:
-            return difftext.CANNOT_DIFF
-        if new_kind == 'file':
-            new_date = self.get_svn_rev_info(difftext.new_tree)
-        elif new_kind is None:
-            new_date = None
-            to_file_id = None
-        else:
-            return difftext.CANNOT_DIFF
-        from_label = '%s%s\t%s' % (difftext.old_label, old_path, old_date or '(revision 0)')
-        to_label = '%s%s\t%s' % (difftext.new_label, new_path, new_date or '(revision 0)')
-        return difftext.diff_text(from_file_id, to_file_id, from_label, to_label)
+    )
+
+from cStringIO import StringIO
+
+from subvertpy import (
+    properties,
+    )
+
+
+class SvnDiffTree(_mod_diff.DiffTree):
+    """Provides a text representation between two trees, formatted for svn."""
+
+    def _get_svn_rev_info(self, tree, file_id):
+        try:
+            rev = tree.inventory[file_id].revision
+        except errors.NoSuchId:
+            return '(revision 0)'
+        if rev is None:
+            return '(working copy)'
+        try:
+            info = self.repository.lookup_revision_id(rev)
+        except errors.NoSuchRevision:
+            return '(working copy)'
+        return '(revision %d)' % info[0][2]
+
+    def _write_contents_diff(self, path, old_version, old_contents, new_version, new_contents):
+        if path is None:
+            return 
+        self.to_file.write("Index: %s\n" % path)
+        self.to_file.write("=" * 67 + "\n")
+        old_label = '%s\t%s' % (path, old_version)
+        new_label = '%s\t%s' % (path, new_version)
+        _mod_diff.internal_diff(old_label, old_contents,
+                                new_label, new_contents,
+                                self.to_file)
+
+    def _write_properties_diff(self, path, old_properties, new_properties):
+        if new_properties is None:
+            return
+        if old_properties is None:
+            old_properties = {}
+        changed = []
+        for name in set(old_properties.keys() + new_properties.keys()):
+            oldval = old_properties.get(name)
+            newval = new_properties.get(name)
+            if oldval != newval:
+                changed.append((name, oldval, newval))
+        if changed == []:
+            return
+        self.to_file.write("Property changes on: %s\n" % path)
+        self.to_file.write("_" * 67 + "\n")
+        for (name, old_value, new_value) in changed:
+            if old_value is None:
+                self.to_file.write("Added: %s\n\t+%s\n" % (name, new_value))
+            elif new_value is None:
+                self.to_file.write("Removed: %s\n\t-%s\n" % (name, old_value))
+            else:
+                self.to_file.write("Changed: %s\n\t-%s\n\t+%s\n" % (name, old_value, new_value))
+
+    def _get_file_properties(self, tree, path, kind, executable):
+        if kind in (None, "directory"):
+            return None
+        ret = {}
+        if executable:
+            ret[properties.PROP_EXECUTABLE] = properties.PROP_EXECUTABLE_VALUE
+        if kind == "symlink":
+            ret[properties.PROP_SPECIAL] = properties.PROP_SPECIAL_VALUE
+        return ret
+
+    def _get_file_contents(self, tree, file_id, path, kind):
+        if kind in (None, "directory"):
+            return None
+        if kind == "symlink":
+            return ["link %s" % tree.get_symlink_target(file_id)]
+        return tree.get_file(file_id).readlines()
+
+    def _show_diff(self, specific_files, extra_trees):
+        iterator = self.new_tree.iter_changes(self.old_tree,
+                                               specific_files=specific_files,
+                                               extra_trees=extra_trees,
+                                               require_versioned=True)
+        has_changes = 0
+        def get_encoded_path(path):
+            if path is not None:
+                return path.encode(self.path_encoding, "replace")
+        for (file_id, paths, changed_content, versioned, parent, name, kind,
+             executable) in iterator:
+            # The root does not get diffed, and items with no known kind (that
+            # is, missing) in both trees are skipped as well.
+            if parent == (None, None) or kind == (None, None):
+                continue
+            oldpath, newpath = paths
+            oldpath_encoded = get_encoded_path(paths[0])
+            newpath_encoded = get_encoded_path(paths[1])
+            old_present = (kind[0] is not None and versioned[0])
+            new_present = (kind[1] is not None and versioned[1])
+            renamed = (parent[0], name[0]) != (parent[1], name[1])
+            old_properties = self._get_file_properties(self.old_tree, oldpath_encoded, kind[0], executable[0])
+            new_properties = self._get_file_properties(self.new_tree, newpath_encoded, kind[1], executable[1])
+            old_version = self._get_svn_rev_info(self.old_tree, file_id)
+            new_version = self._get_svn_rev_info(self.new_tree, file_id)
+
+            if oldpath_encoded == newpath_encoded:
+                if changed_content:
+                    old_contents = self._get_file_contents(self.old_tree, file_id, oldpath_encoded, kind[0])
+                    new_contents = self._get_file_contents(self.new_tree, file_id, newpath_encoded, kind[1])
+                    self._write_contents_diff(oldpath_encoded, old_version, old_contents, new_version, new_contents)
+                self._write_properties_diff(oldpath_encoded, old_properties, new_properties)
+            else:
+                old_contents = self._get_file_contents(self.old_tree, file_id, oldpath_encoded, kind[0])
+                new_contents = self._get_file_contents(self.new_tree, file_id, newpath_encoded, kind[1])
+                self._write_contents_diff(oldpath_encoded, old_version, old_contents, new_version, [])
+                self._write_contents_diff(newpath_encoded, old_version, [], new_version, new_contents)
+                self._write_properties_diff(newpath_encoded, {}, new_properties)
+
+            has_changes = (changed_content or renamed)
+
+        return has_changes
 
 
 class SvnMergeDirective(merge_directive._BaseMergeDirective):
@@ -75,16 +148,14 @@
 
     @classmethod
     def _generate_diff(cls, repository, svn_repository, revision_id, ancestor_id):
-        from bzrlib.diff import DiffText
-        writer = SvnDiffWriter(svn_repository, revision_id)
-        def DiffText_diff(self, file_id, old_path, new_path, old_kind, new_kind):
-            return writer.diff_text(self, file_id, old_path, new_path, old_kind, new_kind)
-        old_DiffText_diff = DiffText.diff
-        DiffText.diff = DiffText_diff
-        patch = merge_directive._BaseMergeDirective._generate_diff(
-            repository, revision_id, ancestor_id)
-        DiffText.diff = old_DiffText_diff
-        return patch
+        tree_1 = repository.revision_tree(ancestor_id)
+        tree_2 = repository.revision_tree(revision_id)
+        s = StringIO()
+        differ = SvnDiffTree.from_trees_options(tree_1, tree_2, s, 'utf8', None,
+            '', '', None)
+        differ.repository = svn_repository
+        differ.show_diff(None, None)
+        return s.getvalue()
 
     @classmethod
     def from_objects(cls, repository, revision_id, time, timezone,
@@ -92,15 +163,12 @@
                      public_branch=None, message=None):
         from bzrlib.plugins.svn.repository import SvnRepository
         submit_branch = _mod_branch.Branch.open(target_branch)
-        if submit_branch.get_parent() is not None:
-            submit_branch = _mod_branch.Branch.open(submit_branch.get_parent())
         if not isinstance(submit_branch.repository, SvnRepository):
             raise errors.BzrError("Not a Subversion repository")
 
         submit_branch.lock_read()
         try:
             submit_revision_id = submit_branch.last_revision()
-            submit_revision_id = _mod_revision.ensure_null(submit_revision_id)
             repository.fetch(submit_branch.repository, submit_revision_id)
             graph = repository.get_graph()
             ancestor_id = graph.find_unique_lca(revision_id,

